@@ -6,23 +6,14 @@ use Mojo::Util 'trim';
 my $PHOTO_RE = qr!\.(?:jpg|png|gif)\b!i;
 my $VIDEO_RE = qr!\.(?:mpg|mpeg|mov|mp4|ogv)\b!i;
 
-sub learn {
-  my ($self, $cb) = @_;
-  my $url = $self->url;
+sub learn_p {
+  my $self = shift;
+  my $url  = $self->url;
   my $type = $url =~ $PHOTO_RE ? 'photo' : $url =~ $VIDEO_RE ? 'video' : 'link';
 
   $self->type($type);
 
-  # Need to learn more from an http request
-  if ($type eq 'link') {
-    $self->SUPER::learn($cb);
-  }
-  else {
-    $self->_learn_from_url;
-    $self->$cb if $cb;
-  }
-
-  return $self;
+  return $type eq 'link' ? $self->SUPER::learn_p : Mojo::Promise->new->resolve($self->_learn_from_url);
 }
 
 sub _learn_from_dom {
@@ -49,6 +40,8 @@ sub _learn_from_dom {
     $tmp =~ s!\s+! !g;
     $self->description(trim $tmp);
   }
+
+  return $self;
 }
 
 1;
